"""
Name: Jason Saladiner Jr.
Date Created: 04/28/2022
Last Updated: 04/28/2022
Purpose: Act as a repository for Shock relations
General Assumptions:
	Attatched Shocks
	Steady Flow
	Adiabatic Flow
Included Functions:
	
"""




def prandtlmeyer(nu=None,Mach=None,gamma=1.4):
	"""
	The relationship between prandl-Meyer angle and mach
	Pass None as variable of Interest
	Returns tuple of Machs. One sonic, One supersonic
	"""
	import sys
	from math import pow,log,sqrt,atan,pi,tan
	
	pop = nu
	
	#Check inputs
	if (pop == None and Mach == None) or (pop !=None and Mach != None):
		sys.exit("Incorrect number of None Variables. Make sure to pass only one None for variable of interest")
	try:
		log(gamma-1)
	except:
		sys.exit("Gamma must be a real number greater than 1")

	if pop == None:
		return (sqrt((gamma+1)/(gamma-1))*atan(sqrt((gamma-1)/(gamma+1)*(Mach**2 - 1)))-atan(sqrt(Mach**2 - 1)))*180/pi
	elif Mach == None:
		pop = pop
		
		C = sqrt((gamma-1)/(gamma+1))
		#Hall Relation
		f = lambda B : tan(C*(pop+atan(B)*180/pi)*pi/180)/C - B
		fprime = lambda B : -(1- C**2)*B**2 / (1 + B**2)
		
		from scipy.optimize import newton
		beta = newton(f,1.3,fprime)
		return sqrt(beta**2+1)


def prandtlequation(nu1=None,nu2=None,delta=None,gamma=1.4):
	"""
	needs docstring
	Needs to be generalized for starting not at delta1 = 0

	"""
	import sys
	from math import pow,log,sqrt,atan,pi,tan
	

	cNone = [nu1,nu2,delta].count(None)
	
	if cNone != 1:
		sys.exit("Incorrect number of None Variables in SalamanderSpace.Relations.Dimensionless.Mach. Make sure to pass only one None for variable of interest")


	if nu1 == None:
		return nu2+delta
	elif nu2 == None:
		return nu1-delta
	elif delta ==None:
		return -(nu2-nu1)





def mach2(Mach1=None,Mach2= None,gamma = 1.4):
	"""
	The relationship between mach1 and mach2 across normal shock
	Pass None as variable of Interest
	"""
	import sys
	from math import pow,log,sqrt,atan,pi
	x = Mach1
	pop = Mach2
	
	#Check inputs
	if (pop == None and x == None) or (pop !=None and x != None):
		sys.exit("Incorrect number of None Variables. Make sure to pass only one None for variable of interest")
	try:
		log(gamma-1)
	except:
		sys.exit("Gamma must be a real number greater than 1")

	if pop == None:
		return sqrt((2+(gamma-1)*x**2)/(2*gamma*x**2 - (gamma-1)))
	elif x == None:
		return sqrt((2+(gamma-1)*pop**2)/(2*gamma*pop**2 - (gamma-1)))

def densityRatio(Mach1 = None,Rho2ToRho1 = None,gamma = 1.4):
	"""
	The relationship between rho2/rho1 to mach1 across normal shock
	Pass None as variable of Interest
	"""
	import sys
	from math import pow,log,sqrt,atan,pi
	
	pop = Rho2ToRho1
	x = Mach1
	#Check inputs
	if (pop == None and x == None) or (pop !=None and x != None):
		sys.exit("Incorrect number of None Variables. Make sure to pass only one None for variable of interest")
	try:
		log(gamma-1)
	except:
		sys.exit("Gamma must be a real number greater than 1")

	if pop == None:
		return (gamma+1)*x**2 / (2 + (gamma-1)*x**2)
	elif x == None:
		return sqrt((2*pop )/( (gamma+1) - pop*(gamma-1)) )

def pressureRatio(Mach1 = None,P2ToP1 = None,gamma = 1.4):
	"""
	The relationship between p2/p1 to mach1 across normal shock
	Pass None as variable of Interest
	"""
	import sys
	from math import pow,log,sqrt,atan,pi
	
	pop = P2ToP1
	x = Mach1
	#Check inputs
	if (pop == None and x == None) or (pop !=None and x != None):
		sys.exit("Incorrect number of None Variables. Make sure to pass only one None for variable of interest")
	try:
		log(gamma-1)
	except:
		sys.exit("Gamma must be a real number greater than 1")

	if pop == None:
		return 1 + (2*gamma) / (gamma + 1) * (x**2 - 1)
	elif x == None:
		return sqrt( (pop-1)*(gamma+1)/(2*gamma) + 1 )

def temperatureRatio(Mach1 = None,T2ToT1 = None,gamma = 1.4):
	"""
	The relationship between T2/T1 to mach1 across normal shock
	Pass None as variable of Interest
	"""
	import sys
	from math import pow,log,sqrt,atan,pi
	
	pop = T2ToT1
	x = Mach1
	#Check inputs
	if (pop == None and x == None) or (pop !=None and x != None):
		sys.exit("Incorrect number of None Variables. Make sure to pass only one None for variable of interest")
	try:
		log(gamma-1)
	except:
		sys.exit("Gamma must be a real number greater than 1")


	C = 2*(gamma-1)*pow(gamma+1,-2)
	if pop == None:
		return 1 + C*pow(x,-2)*(x**2-1)*(gamma*x**2+1)
	elif x == None:
		f = lambda m2 : (1-pop)*m2 + C*(m2-1)*(gamma*m2 + 1)
		fprime = lambda m2 : (1-pop) + C*(2*gamma*m2 - (gamma-1))
		from scipy.optimize import newton
		m2 = newton(f,1.1,fprime)
		return sqrt(m2)


def Astar_A(As_A = None,Mach = None,gamma = 1.4):
	"""
	The relationship between dynamic to static pressure and Mach
	Pass None as variable of Interest
	Returns tuple of Machs. One sonic, One supersonic
	"""
	import sys
	from math import pow,log
	
	pop = As_A
	
	#Check inputs
	if (pop == None and Mach == None) or (pop !=None and Mach != None):
		sys.exit("Incorrect number of None Variables. Make sure to pass only one None for variable of interest")
	try:
		log(gamma-1)
	except:
		sys.exit("Gamma must be a real number greater than 1")
	
	#if pop is variable of interest
	if pop == None:
		return pow((gamma+1)/2,(gamma+1)/(2*gamma-2))*Mach*pow(1+(gamma-1)/2*Mach**2,(gamma+1)/(2-2*gamma))

	elif Mach == None:
		from scipy.optimize import newton
		C = pow((gamma+1)/2,(gamma+1)/(2*gamma-2))
		E = - (gamma+1) / (2*gamma - 2)

		f = lambda M: C*M*pow(1+(gamma-1)/2 * pow(M,2),E) - pop
		fprime = lambda M: C*pow(1+(gamma-1)/2*pow(M,2),E) + C*E*(gamma-1)*pow(M,2)*pow(1 + (gamma - 1) / 2 *pow(M,2),E-1)
		return newton(f,.5,fprime),newton(f,1.5,fprime)

		
	else:
		sys.exit("Could not find Variable of interest in Relations.Astar_A()")


def V_astar(V_astar = None,Mach=None,gamma = 1.4):
	"""
	The relationship between V/a* and Mach
	Pass None as variable of Interest
	"""
	import sys
	from math import pow,log,sqrt
	
	pop = V_astar
	
	#Check inputs
	if (pop == None and Mach == None) or (pop !=None and Mach != None):
		sys.exit("Incorrect number of None Variables. Make sure to pass only one None for variable of interest")
	try:
		log(gamma-1)
	except:
		sys.exit("Gamma must be a real number greater than 1")

	if pop == None:
		vasq = (gamma+1)/2 * Mach**2 * pow(1+(gamma-1)/2 * Mach **2,-1)
		return sqrt(vasq)
	elif Mach == None:
		msq = 2/(gamma+1)*pop**2 * pow(1- (gamma-1)/(gamma+1)*pop**2,-1)
		return sqrt(msq)

####Last Column. Finding Mach from P02ToP1 is hard. Needs to be figured out####
#def RayleighPitot(Mach1 = None,P02ToP1 = None,gamma = 1.4):
#	"""
#	The relationship between P02/P1 to mach1 across normal shock
#	Used for Pitot tubes only
#	Pass None as variable of Interest
#	"""
#	import sys
#	from math import pow,log,sqrt,atan,pi
	
#	pop = P02ToP1
#	x = Mach1
#	#Check inputs
#	if (pop == None and x == None) or (pop !=None and x != None):
#		sys.exit("Incorrect number of None Variables. Make sure to pass only one None for variable of interest")
#	try:
#		log(gamma-1)
#	except:
#		sys.exit("Gamma must be a real number greater than 1")


#	C = pow((gamma+1)/2,gamma/(gamma-1))
#	G = lambda b : 2*gamma / (gamma + 1) * pow(b,1-gamma) - (gamma-1)/(gamma+1)*pow(b,-gamma)
#	if pop == None:
#		return C*pow(G(x**2),-1/(gamma-1))
#	elif x == None:
#		f = lambda B: C*pow(G(B),-1/(gamma-1)) - pop
#		fprime = lambda B: C*(gamma/(gamma+1) * (2*pow(B,-gamma) + pow(B,-gamma-1)) * pow(G(B),-gamma))
		
#		print(G(.1))
#		print(f(.1))
#		print(fprime(.1))
#		from scipy.optimize import newton
#		beta = newton(f,15,disp = True)
#		return sqrt(beta)


"""
Name: Jason Saladiner Jr.
Date Created: 04/28/2022
Last Updated: 04/28/2022
Purpose: Create classes for useful tables
General Assumptions:

Included Functions/Submodules:
	class NACA1135
	class StandardAtmosphere
"""
import numpy as np







class NACA1135():
	

	def _createRow_(self,Mach):
		"""
		creates the row in the same fashion as NACA 1135 for given Mach
		"""
		
		from SalamanderSpace.Relations import Isentropic as I
		from math import pow,inf,asin,pi

		self.row = []
		self.M = Mach


		##Subsonic##
		
		self.row.append(self.M)
		self.row.append(pow(I.P0_P(Mach = self.M,gamma=self.g),-1))
		self.row.append(pow(I.den0_den(Mach=self.M,gamma = self.g),-1))
		self.row.append(pow(I.T0_T(Mach=self.M,gamma=self.g),-1))
		self.row.append(pow(abs(pow(self.M,2)-1),1/2))
		self.row.append(I.q_P(Mach=self.M,gamma=self.g)*pow(I.P0_P(Mach=self.M,gamma=self.g),-1))
		
		if Mach <= 1e-8:
			self.row.append(inf)
		else:
			from SalamanderSpace.Relations import Astar_A as A
			self.row.append(A(Mach=self.M,gamma = self.g))
		from SalamanderSpace.Relations import V_astar as V
		self.row.append(V(Mach=self.M,gamma = self.g))

		if self.M <1:
			self.row.append(None)
			self.row.append(None)
			self.row.append(None)
			self.row.append(None)
			self.row.append(None)
			self.row.append(None)
			return self.row

		from SalamanderSpace.Relations import ShockRelations as S
		self.row.append(S.prandtlmeyer(Mach = self.M,gamma = self.g))
		self.row.append(asin(1/self.M)*180/pi)
		self.row.append(S.mach2(Mach1 = self.M,gamma = self.g))
		self.row.append(S.pressureRatio(Mach1=self.M,gamma=self.g))
		self.row.append(S.densityRatio(Mach1=self.M,gamma=self.g))
		self.row.append(S.temperatureRatio(Mach1=self.M,gamma=self.g))

		###Need to add Rayleigh pitot ratios###
		return self.row
	def buildTable(self,n=10):
		"""
		creates the table from Mach 0->Mach n (default 10) at intervals of .01 Mach
		"""
		self.n = n
		#from numpy import linspace,array
		self.M1 = np.linspace(0,n,100*n+1)
		self.tab = []
		for i in range(len(self.M1)):
			self.tab.append(self._createRow_(self.M1[i]))

		self.Table =  np.array(self.tab,dtype = object)

		


	def getRow(self,**given):
		"""
		Returns NACA 1135 row for a given variable
		given must be in from string Varible = float Value
		Varibable names include:
			Mach1, isenPressure, isenDensity, isenTemperature, Beta, dynTotalPressure, AreaRatio, VaStar, 
			nu, mu, Mach2, shockPressure, shockDensity, shockTemperature, ***Pitot Values Not Added Yet*** 
		"""
		import SalamanderSpace.Relations as ssR
		import SalamanderSpace.Relations.Isentropic as I
		import SalamanderSpace.Relations.ShockRelations as S

		from math import sqrt, abs, sin
		self.varDict = {"mach1" : 0, "isenpressure" : I.P0_P, "isendensity" : I.den0_den, "isentemperature" : I.T0_T, "beta" : 4, "dyntotalpressure" : I.q_P,"arearatio" : ssR.Astar_A,"nu" : S.prandtlmeyer, "mu" : 9, "mach2" : S.mach2, "shockpressure": S.pressureRatio, "shockdensity" : S.densityRatio, "shocktemperature":S.temperatureRatio}
		
		for var,value in given.items():
			if self.varDict[var] == 0 :
				self.m = var
			elif self.varDict[var] == 4:
				self.m = sqrt(abs(value**2 - 1))
			elif self.varDict[var] == 9:
				self.m = 1 / (sin(value*pi/180))
			elif var.lower()[:2] == "is" or var.lower()[:2] == "dy" or var.lower()[:2] == "ar":
				self.m = self.varDict[var](value**-1,Mach=None,gamma=1.4)

		return self._createRow_(self.m)



		#self.roundM = round(Mach,2)
		
		#if self.roundM >=0 and self.roundM <=(len(self.Table)-1)/100:
		#	self.ind = np.where(abs(self.Table[:,0] - self.roundM) <=1e-6)
		#	self.row =  self.Table[self.ind]
		#	if self.roundM < 1:
		#		return self.row[0,0:8]
		#	return self.row[0]
		#elif self.roundM >= (len(self.Table[:,0])-1)/100:
		#	return self._createRow_(self.roundM)
		#else:
		#	import sys
		#	sys.exit("Mach is negative which is not possible.")


	#def findElement(self,*var,**given):
	#	"""
	#	def findElement will return the value of a given set of *variables (passed as strings) with some **given (string = num)
	#	Varibable names include:
	#		Mach1, isenPressure, isenDensity, isenTemperature, Beta, dynTotalPressure, AreaRatio, VaStar, 
	#		nu, mu, Mach2, shockPressure, shockDensity, shockTemperature, ***Pitot Values Not Added Yet*** 
	#	"""
	#	self.var = str(var).lower()
	#	self.varDict = {"mach1" : 0, "isenpressure" : 1, "isendensity" : 2, "isentemperature" : 3, "beta" : 4, "dyntotalpressure" : 5,
	#			  "arearatio" : 6, "vastar" : 7, "nu" : 8, "mu" : 9, "mach2" : 10, "shockpressure": 11, "shockdensity" : 12, "shocktemperature":13}
	#	self.indx = []
	#	self.value = []
	#	for i in given:
	#		self.indx.append(self.varDict[i[0].lower()])
	#		self.value.append(i[1])

	def printNames():
		"""
		0-Mach1, 1-isenPressure, 2-isenDensity, 3-isenTemperature, 4-Beta, 5-dynTotalPressure, 6-AreaRatio, 7-VaStar, 8-nu, 9-mu, 
		10-Mach2, 11-shockPressure, 12-shockDensity, 13-shockTemperature

		Isen: p/pt
		shock: 2/1
		"""
	def __init__(self,gamma = 1.4):
		
		self.g = gamma


class standardAtmosphere():



	def _setBoundaries_(self,bondaries):
		"""
		Sets the boundaries of the atmospheric layers\n
		Input:\n
			\tlist boundaries = list of lists with each row corresponding to base of each atmospheric layer\n
			\t[base geopotential alt,Base Temp,Base Pressure,Base Density]\n
			\n
			\n
		**Note: Units should remain constant and are not check by this class.** \n
			\tfor SI: use meters, K, pascals, kg/m^3\n
			\tfor English Eng Units: use ft, Rankie, Atm, slugs/ft^3
		"""

	def _tempProfile_(self):
		"""
		Create a temperature profile from the given model\n
		Updates self.model to include lapse rate (between base temp and base pressure)
		"""


		
			
		#print(self.model)
		for self.i in range(len(self.model)-1):
			#print(self.model[i+1][1],self.model[i][1],self.model[i+1][0],self.model[i][0])
			print(self.i+1)

			self.lr = (self.model[self.i+1][1] - self.model[self.i][1])/(self.model[self.i+1][0]-self.model[self.i][0])

			self.model.insert(2,self.lr)
	
	

	def _iso_(self,T,dh):
		"""
		find the ratio of final to inital pressure and temp in isothermal region
		return tuple 
		"""
		from math import exp
		self.rat = exp(-(self.g*dh)/(self.R*T))
		return self.rat,self.rat

	def _grad_(self,T,Ti,lr):
		"""
		find the ratio of final to inital pressure and temp in gradient region
		return tuple = p/pi, rho/rhoi
		"""
		from math import pow
		self.prat = pow(T/Ti,-self.g/lr/self.R)
		self.rhorat = pow(T/Ti,-(self.g/lr/self.R+1))

		return self.prat,self.rhorat


	def fromGP(self,height):
		"""
		Get the standard atmosphere values at a given geopotential alt\n
		**Inputs**\n
		-- float height = geopotential altitude above mean sea level\n
		**Returns**\n
		-- list atmosphere = [height,Temp,pressure,density]
		"""
		self.atmosphere = []
		self.atmosphere.append(height)
		for i in range(len(self.model)):
			if height - self.model[i][0] < 1e-6:
				self.atmosphere.append(self.model[i][1])
				self.atmosphere.append(self.model[i][3])
				self.atmosphere.append(self.model[i][4])
				return self.atmosphere
			elif height < self.model[i][0]:
				self.dh = height - self.model[i-1][0]
				self.tnew = self.model[i-1][1] + self.model[i-1][2] * self.dh
				
				self.atmosphere.append(self.tnew)

				#check if iso or gradient
				if self.model[i-1][2] < 1e-6:
					self.rats = self._iso_(self.tnew,self.dh)
				else:
					self.rats = self._grad_(self.tnew,self.model[i-1][1],self.model[i-1][2])

				self.atmosphere.append(self.rats[0] * self.model[i-1][3])
				self.atmosphere.append(self.rats[1] * self.model[i-1][4])
				return self.atmosphere
		
		self.i = len(self.model)
		self.dh = height - self.model[i-1][0]
		self.tnew = self.model[i-1][1] + self.model[i-1][2] * self.dh
				
		self.atmosphere.append(self.tnew)

		#check if iso or gradient
		if self.model[i-1][2] < 1e-6:
			self.rats = self._iso_(self.tnew,self.dh)
		else:
			self.rats = self._grad_(self.tnew,self.model[i-1][1],self.model[i-1][2])

		self.atmosphere.append(self.rats[0] * self.model[i-1][3])
		self.atmosphere.append(self.rats[1] * self.model[i-1][4])
		return self.atmosphere


	def fromGM(self,height):
		"""
		Get the standard atmosphere values at a given geometric alt\n
		**Inputs**\n
		-- float height = geometric altitude above mean sea level\n
		**Returns**\n
		-- list atmosphere = [height,Temp,pressure,density]
		"""

		#self.gp = 

	def __init__(self,g,**kwargs):
		"""
		Create a standard atmosphere for estimations using US 1976 Standard Atmosphere Equations\n
		Can change to different inital conditions/atmoshpere boundaries\n
			\tDefaults to ISA 1976
		**Inputs:**\n
		g = gravitational constant at mean sea level\n
		\n
		
		kwargs\n
		--string units = used units [SI or EE]     *Currently supports SI and English Eng. Units (Defaults to SI)*\n
		--float gamma = ratio of specific heats\n
		--list model = list of list setting boundaries \tfor SI: use meters, K, pascals, kg/m^3\n
		"""
		self.g = g

		##KWARGS
		
		try:			#Units
			self.units = kwargs["units"]
		except:
			self.units = "SI"
		try:			#gamma
		   self.gamma = kwargs["gamma"]
		except:
			self.gamma = 1.4
		try:			#Model
			self.model = kwargs["model"]
		except:
			self.ISA1976 = [[-610,292.15,108900,1.2985],[11000,216.65,22632,0.3639],[20000,216.65,5474.9,0.0880],[32000,228.65,868.02,0.0132],[47000,270.65,110.91,0.0020],[51000,270.65,66.939,0],[71000,214.65,3.9564,0],[84852,186.87,0.3734,0]]
			self.model = self.ISA1976
		

		self._tempProfile_()


"""
Name: Jason Saladiner Jr.
Date Created: 04/28/2022
Last Updated: 04/28/2022
Purpose: 
General Assumptions:

Included Functions/Submodules:
	
"""
##Documentation
## In command line:

##Doesn't work bc webbrowser wont open chrome#### To create documentation html files
##Doesn't work bc webbrowser wont open chrome#### > pyton -m pdoc --html -o SalamanderSpace\Documentation\html SalamanderSpace

## To create local host 
### > pip install pdoc3
### > python -m pdoc --http : SalamanderSpace
## Go to chrome and type in search bar : "localhost:8080"


__all__ = ["FileIO", "Shocks", "Tables", "vlm"]

















def variationOfProperties(entropy=None,entropyAtEnthalpy=None,pressure=None,R=.28716):
	"""
	Finding the variation of Properties (s,h,p) for a gas above a standard pressure (1 bar for air)
	entropyAtEnthalpy is usually based on tabulated values.
	pressure is given in bars and entropys are given in kJ/KgK by default (for R = .28716). Change R for different units.
	Pass None as variable of Interest
	"""
	import sys
	from math import pow,log,exp
	
	s = entropy
	sh = entropyAtEnthalpy
	p = pressure
	#count of the number of None Types
	cNone = [s,sh,p].count(None)
	
	if cNone != 1:
		sys.exit("Incorrect number of None Variables in SalamanderSpace.Relations.Dimensionless.Mach. Make sure to pass only one None for variable of interest")

	if s == None:
		return sh-R*log(p)
	elif sh == None:
		return s + R*log(p)
	elif p == None:
		return exp(-(s-sh)/R)



def Sutherland(T,mu_ref = 1.716e-5, T_ref = 273.15, S_mu = 110.6):
	"""
	Sutherland's law finds the dynmic viscosity at a given temperature based on reference values.
	Default values are for dry air.

	Returns float mu
	"""

	##TODO Find a proper home for this
	from math import pow
	Trat = pow(T/T_ref,3/2)
	Tadd = (T_ref + S_mu) / (T + S_mu)

	return mu_ref*Trat*Tadd



"""
Name: Jason Saladiner Jr.
Date Created: 04/29/2022
Last Updated: 04/29/2022
Purpose: Act as a repository for Isentropic Relations
General Assumptions:

Included Functions:
    def Mach(velocity,a,Mach)
    def Reynolds(density,velocity,length,dynamicViscosity,Reynolds)
"""


def Mach(velocity=None,a=None,Mach=None):
	import sys
	
	#count of the number of None Types
	cNone = [velocity,a,Mach].count(None)
	
	if cNone != 1:
		sys.exit("Incorrect number of None Variables in SalamanderSpace.Relations.Dimensionless.Mach. Make sure to pass only one None for variable of interest")

	if Mach == None:
		return velocity/a
	elif a == None:
		return velocity / Mach
	elif velocity == None:
		return a*Mach

def Reynolds(density = None,flowSpeed = None,Length = None,dynamicViscosity = None,ReynoldsNumber = None):
	import sys
	from math import prod
	var = [density,flowSpeed,Length,dynamicViscosity,ReynoldsNumber]
	
	if var.count(None) != 1:
		sys.exit("Incorrect number of None Variables in SalamanderSpace.Relations.Dimensionless.Reynolds. Make sure to pass only one None for variable of interest")

	interest = var.index(None)

	if interest == 3 or interest == 4:
		return prod(var[0:3]) / var[2*interest-7]
	else:
		rhs = var[0:3]
		return prod(var[3:])/(rhs[interest-1]*rhs[interest-2])


"""
TODO:

Documentation at top of modules
Write read/split functions for floats,int, "strings" (mixed)
		Test bc csvs are weird
		Code other delimiters
Create list of functions (aka plan out packages)
"""


def readFile(name):
	"""
	This function will read the contents of a file by a given name in the "Files" folder of this function
	Input: String name = the name of the file to be read with file extension
	Output: array lines = array of each of the lines
	"""

	# get the path of the input file relative to this file
	import os
	#get teh current path and the current script name
	curPath = os.path.abspath(__file__)
	scriptName = os.path.basename(__file__)

	#change the slashes
	curPath = curPath.replace("\\","//")
	#change path to be towards the file to read
	curPath = curPath.replace(scriptName,"Files//"+name)

	#open the file
	f = open(curPath,"r")
	#read the file
	lines = f.readlines()
	#close the file
	f.close()
	
	#returns unclean array of each of the lines
	return lines


def splitCSV(lines):
	"""
	This function will take in an array from a csv file and split it into 'cells'
	input: array lines = an array of data delimited by a comma and each line ends with a \n
	output: 2d array data = array of arrays that is the data of each 'cell'
	"""

	#initialize the return array data
	data = []
	
	#create the 2d data array of clean data
	for i in range(0,len(lines)):
		data.append(lines[i].strip("\n").split(","))

	#return the proper array
	return data


def filetoarray(file,**kwargs):
	"""
	filetoarray will return 2d array from file
	Inputs:
		string file = path of file to seperate
		**kwars:
			tuple(int) bounds = tuple of ints with bounds (default 0,n)
			string delim = delimiter to split each line (default ",")
			string dataType = type of data to change cells into. (default string))
				Options:
					string = string,str,s
					int = int, i
					complex = complex, com, c
					float = float, f
			
	"""
	
	#Open reader
	f = open(file,"r")


	##Setting reading options
	if kwargs["dataType"] is not None:
		dataType = kwargs["dataType"]
	else:
		dataType = "s"

	if kwargs["delim"] is not None:
		delim = kwargs["delim"]
	else:
		delim = ","
	

	#reading file
	lines = f.readlines()
	
	data = []
	for i in range(kwargs["bounds"][0],kwargs["bounds"][0]):
		splitline =lines[i].strip("\n").split(delim)
		templine = []

		##Change element type
		for j in range(len(splitline)):
			try:
				if dataType.lower() == "s" or dataType.lower() == "str" or dataType.lower() == "string":
					templine.append(str(splitline[j]))
				elif dataType.lower() == "f" or dataType.lower() =="float":
					templine.append(float(splitline[j]))
				elif dataType.lower() == "i" or dataType.lower() == "int":
					templine.append(int(splitline[j]))
				elif dataType.lower() =="c" or dataType.lower() == "com" or dataType.lower() == "complex":
					templine.append(complex(splitline[j]))
				else:
					print("ERROR:dataType not recognized")
			except:
				templine.append(splitline[j])

		#append to master file
		data.append(templine)

	return data


def timer(toTest,correct=None,**kwargs):
    from datetime import datetime
    result = None
    #Start Timer
    start = datetime.now()

    
    result = toTest(**kwargs)


    #Stop Timer
    finish = datetime.now()
    #get elapsed time
    elapse = finish-start


    print("\tTest Finished")
    print("=====================================")
    print("Time Elapsed:\t",elapse," sec")
    print("Given Result:\t",result)
    print("Correct Result:\t", correct)


"""
Name: Jason Saladiner Jr.
Date Created: 04/28/2022
Last Updated: 04/28/2022
Purpose: Act as a repository for Isentropic Relations
General Assumptions:
	Flow is...
	-Steady 
	-Adiabatic
	-Reversible

Included Functions:
	def P0_P(StaticPressureToPressure,Mach,Gamma)
	def den0_den(StaticDensityToDensity,Mach,Gamma)
	def T0_T(StaticTemperatureToTemperature,Mach,Gamma)
"""


def P0_P (StaticPressureToPressure=None,Mach=None,gamma = 1.4):
	"""
	The relationship between static pressure over pressure to mach at a given ratio of specific heats
	Pass None as variable of Interest
	"""
	import sys
	from math import pow,log
	
	pop = StaticPressureToPressure
	
	#Check inputs
	if (pop == None and Mach == None) or (pop !=None and Mach != None):
		sys.exit("Incorrect number of None Variables. Make sure to pass only one None for variable of interest")
	try:
		log(gamma-1)
	except:
		sys.exit("Gamma must be a real number greater than 1")
	
	#if pop is variable of interest
	if pop == None:
		try:
			return pow(1+(gamma-1)/2*pow(Mach,2),gamma/(gamma-1))
		except:
			sys.exit("StaticPressureToPressure was the variable of interest. Issue finding. Make sure Mach is a float")
	elif Mach == None:
		try:
			return pow(2/(gamma-1)*(pow(pop,(gamma-1)/gamma)-1),1/2)
		except:
			sys.exit("Mach was the variable of interest. Issue finding. Make sure StaticPressureToPressure is a float")
	else:
		sys.exit("Could not find Variable of interest")

def den0_den(StaticDensityToDensity = None,Mach = None,gamma = 1.4):
	"""
	The relationship between static density over density to mach at a given ratio of specific heats
	Pass None as variable of Interest
	"""
	import sys
	from math import pow,log
	
	pop = StaticDensityToDensity
	
	#Check inputs
	if (pop == None and Mach == None) or (pop !=None and Mach != None):
		sys.exit("Incorrect number of None Variables. Make sure to pass only one None for variable of interest")
	try:
		log(gamma-1)
	except:
		sys.exit("Gamma must be a real number greater than 1")
	
	#if pop is variable of interest
	if pop == None:
		try:
			return pow(1+(gamma-1)/2*pow(Mach,2),1/(gamma-1))
		except:
			sys.exit("StaticDensityToDensity was the variable of interest. Issue finding. Make sure Mach is a float")
	elif Mach == None:
		try:
			return pow(2/(gamma-1)*(pow(pop,(gamma-1))-1),1/2)
		except:
			sys.exit("Mach was the variable of interest. Issue finding. Make sure StaticDensityToDensity is a float")
	else:
		sys.exit("Could not find Variable of interest")

def T0_T(StaticTemperatureToTemperature = None,Mach= None,gamma = 1.4):
	"""
	The relationship between static Temperature over Temperature to mach at a given ratio of specific heats
	Pass None as variable of Interest
	"""
	import sys
	from math import pow,log
	
	pop = StaticTemperatureToTemperature
	
	#Check inputs
	if (pop == None and Mach == None) or (pop !=None and Mach != None):
		sys.exit("Incorrect number of None Variables. Make sure to pass only one None for variable of interest")
	try:
		log(gamma-1)
	except:
		sys.exit("Gamma must be a real number greater than 1")
	
	#if pop is variable of interest
	if pop == None:
		try:
			return 1 + (gamma-1)/2 *pow(Mach,2)
		except:
			sys.exit("StaticTemperatureToTemperature was the variable of interest. Issue finding. Make sure Mach is a float")
	elif Mach == None:
		try:
			return pow(2/(gamma-1)*(pop-1),1/2)
		except:
			sys.exit("Mach was the variable of interest. Issue finding. Make sure StaticTemperatureToTemperature is a float")
	else:
		sys.exit("Could not find Variable of interest")

def q_P(dynamicToStaticPressure = None, Mach = None, gamma =1.4):
	"""
	The relationship between dynamic to static pressure and Mach
	Pass None as variable of Interest
	"""
	import sys
	from math import pow,log
	
	pop = dynamicToStaticPressure
	
	#Check inputs
	if (pop == None and Mach == None) or (pop !=None and Mach != None):
		sys.exit("Incorrect number of None Variables. Make sure to pass only one None for variable of interest")
	try:
		log(gamma-1)
	except:
		sys.exit("Gamma must be a real number greater than 1")
	
	#if pop is variable of interest
	if pop == None:
		return gamma/2*pow(Mach,2)
	elif Mach == None:
		return pow(2*pop/gamma,1/2)
		
	else:
		sys.exit("Could not find Variable of interest in Relations.Isentropic.q_P()")




"""
will be used for vlm hw and possible other extensions 
"""


class flatWingGeometry():
	"""
	This class will contain the geomertry of a flat plate at a specific resolution. The geomertry is created by transforming a flat rectangle in numerous methods.
	"""

	def plot3d(self):
		"""
		plots the current iteration of the wing using matplotlib
		"""
	
		import matplotlib.pyplot as plt
		fig = plt.figure()
		ax = fig.add_subplot(111, projection = '3d')
		
		ax.scatter(self.points[:,0],self.points[:,1],self.points[:,2])
		plt.xlim([0,.4])
		plt.ylim([-.5,.5])
		ax.set_zlim3d(-.1,.1)
		#plt.show()
	
	def Dihedral(self,gamma):
		import numpy as np
		from math import sin,cos


		self.g = gamma
		self.temp = self.points[0][1]
		if self.temp < 0:
			self.g = - gamma
		self.dihed = np.array([1,0,0,0,cos(self.g),-sin(self.g),0,sin(self.g),cos(self.g)]).reshape((3,3))


		for i in range(len(self.points[:,0])):
			if not self.points[i][1] == self.temp:
				self.g = gamma
				if self.points[i][1] <0:
					self.g = -gamma
				self.dihed = np.array([1,0,0,0,cos(self.g),-sin(self.g),0,sin(self.g),cos(self.g)]).reshape((3,3))
			self.points[i] = np.matmul(self.dihed,self.points[i])
			self.temp = self.points[i][1]
		

	def LESweep(self,lamb):
		self.l = lamb
		import numpy as np
		from math import tan
		self.shear = np.array([1,tan(self.l),0,0,1,0,0,0,1]).reshape((3,3))
		for i in range(len(self.points[:,0])):
			if self.points[i][1] < 0:
				self.shear[0][1] = -tan(self.l)
			else:
				self.shear[0][1] = tan(self.l)
			self.points[i] = np.matmul(self.shear,self.points[i])


	def _washTrans_(self,theta):
		from math import cos,sin

		self.line0 = [cos(theta),0,sin(theta)]
		self.line1 = [0,1,0]
		self.line2 = [-sin(theta),0,cos(theta)]

		return [self.line0,self.line1,self.line2]


	def linWashout(self,washout):
		"""
		applies a linear washout to a defined wing. washout should be provided in radians
		"""
		import numpy as np
		
		theta = lambda y : washout * (1 - abs(y))
		
		self.tempy = self.tempy = self.points[0][1]
		self.tempTrans = self._washTrans_(theta(self.tempy))

		for i in range(len(self.points[:,0])):
			if not self.points[i][1] == self.tempy:
				self.tempTrans = self._washTrans_(theta(self.points[i][1]))
				self.tempy = self.points[i][1]
			
			self.points[i] = np.matmul(self.tempTrans,self.points[i])
	

	
	
		
		
	def _cofy_(self,y):
		"""
		c(y) nondimensionalized (scaled by b)
		"""
		
		
		self.cr = 2 / (self.AR*(1+self.taper))

		return self.cr*(1-2*(1-self.taper)*abs(y))



	def __init__(self,AR,taper,yrange = (-.5,.5),res = (.25,.1)):
		"""
		arguments:
		span/b = m
		area/s = m^2
		taper = dimensionless
		sweep = rad
		AR = dimensionless
		
		
		###TODO make it allow swept leading edge
		
		"""
		import numpy as np
		self.AR = AR
		self.taper = taper
		self.xr = res[0]
		self.yr = res[1]
		self.ymax = yrange[1]
		self.z = 0.

		self.LE = 0.


		self.points = []

		#creates a list of y points. Will always have the start and stop values of yrange even if the resolution is smaller inbetween last points
		self.ypoints = np.append(np.arange(yrange[0],yrange[1],self.yr),yrange[1])
		
		for y in self.ypoints:
			self.curC = self._cofy_(y)
			
			self.xpoints = np.append(np.arange(self.LE,self.LE + self.curC,self.curC*self.xr),self.curC+self.LE)

			for x in self.xpoints:
				self.points.append([x,y,self.z])
		

		self.points = np.array(self.points)
		



"""
Name: Jason Saladiner Jr.
Date Created: 04/28/2022
Last Updated: 04/28/2022
Purpose: Create classes for full shock regiments
General Assumptions:

Included Functions/Submodules:
	class NormalShock
	class ObliqueShock
"""


####Do we need??? What was use case/outputs
class NormalShock:





	def __init__(self,Mach,gamma=1.4):
		"""
		NormalShock will create a shock and apply normal shock relations across it
		"""
		
		
		self.M = Mach
		self.g = gamma

		



class methodOfCharacteristics():


	def setDeflection(self,delta,N,height):
		"""
		set the deflection characteristics of expansion fan
		float delta = deflection of flow (should be negative)
		int N = number of waves
		float height = height of tunnel



		****Needs to write out units****
		"""
		import sys
		self.d = delta
		self.n = N
		self.h = height

		
		from numpy import linspace
		self.deflections = linspace(0,self.d,self.n)

	def _initalizeWaves_(self):
		import SalamanderSpace.Relations.ShockRelations as S
		
		self.waves = []
		self.waves.append(ExpansionWave(self.M,self.deflections[0],self.g))

		
		for i in range(1,self.n):
			self.deli = self.deflections[i]
			self.waves.append(ExpansionWave(S.prandtlmeyer(S.prandtlequation(self.waves[0].nu,None,self.deli,self.g),None,self.g),self.deli,True,self.g))
		for i in range(self.n):
			self.waves[i].setStart(0,0)
	
			
	def _findIntersect_(self,y1,y2):
		"""
		given two lambda functions
		"""
	
		self.f = lambda x: y1(x) - y2(x)
		from scipy.optimize import newton
		self.xroot = newton(self.f,1)
		self.yroot = y1(self.xroot)

		return (self.xroot,self.yroot)

	def _whichWave_(self,index):
		self.pi = -1
		self.ni = -1
		for i in range(len(self.posIndex)):
			if self.posIndex[i].count(index) !=0:
				self.pi = i
		for i in range(len(self.negIndex)):
			if self.negIndex[i].count(index) !=0:
				self.ni = i
		return (self.ni,self.pi)


	def runMethod(self):
		"""
		Run the method of characteristics
		"""
		import numpy as np


		
		self._initalizeWaves_()

		self.points = []
		self.points.append((0,0))

		self.wallindx = []
		self.wallindx.append(1)
		for i in range(self.n,1,-1):
			self.wallindx.append(self.wallindx[len(self.wallindx)-1] + i)

		self.totPoints = int(self.n * (self.n+1) / 2)

		self.posIndex = []
		self.negIndex = []
		for i in range(1,self.n+1):
			self.curWave = []
			self.curWave.append(i)
			for j in range(1,i):
				self.curWave.append(self.curWave[j-1] + self.n - j)
			self.posIndex.append(self.curWave)

		#print(self.posIndex)
		for i in range(self.n+1,self.n*2+1):
			
			self.curWave = []
			self.curWave.append(self.posIndex[i-self.n-1][-1])
			for j in range(1,2*self.n - i + 1):
				
				self.curWave.append(self.curWave[j-1] + 1)
			self.negIndex.append(self.curWave)

		
		#print(self.negIndex)
		self.ywall = lambda x: self.h
		
		for i in range(1,self.totPoints+1):
			if i in self.wallindx:
				self.nw, self.pw = self._whichWave_(i)
				self.p = self._findIntersect_(self.ywall,self.waves[self.pw].line())
				self.points.append(self.p)

				self.waves.append(ExpansionWave(self.waves[self.pw].M,self.waves[self.pw].d,False,self.waves[self.pw].g))
				self.waves[len(self.waves)-1].setStart(self.p[0],self.p[1])
			else:
				self.nw, self.pw = self._whichWave_(i)
				self.nw = self.nw+self.n
				
				self.p = self._findIntersect_(self.waves[self.nw].line(),self.waves[self.pw].line())
				self.points.append(self.p)

				self.newNu = 1/2* (self.waves[self.nw].K + self.waves[self.pw].K)
				self.newD = 1/2* (self.waves[self.nw].K - self.waves[self.pw].K)

				self.waves[self.nw].updateWave(self.newNu,self.newD)
				self.waves[self.nw].setStart(self.p[0],self.p[1])

				self.waves[self.pw].updateWave(self.newNu,self.newD)
				self.waves[self.pw].setStart(self.p[0],self.p[1])
		#print(self.points)
		




	def __init__(self,Mach,gamma=1.4):
		self.M = Mach
		self.g = gamma



class ExpansionWave():

	def line(self):
		from math import tan, pi
		
		if self.phi >= 0:
			
			self.y = lambda x: (x-self.start[0])*tan(self.phi*pi/180) + self.start[1]
		else:
			self.y = lambda x: (x-self.start[0])*tan(self.phi*pi/180) + self.start[1]
		return self.y
	def setStart(self,x,y):
		
		self.start = (x,y)
		

	def updateWave(self,NewNu,NewDelta):
		self.nu = NewNu
		self.d = NewDelta

		import SalamanderSpace.Relations.ShockRelations as S

		self.M = S.prandtlmeyer(self.nu,None,self.g)
		from math import asin,pi
		self.mu = asin(1/self.M) *180/pi 
		
		self.phi = self.d + self.mu
		
		if self.pos == False:
			self.phi = self.d - self.mu
		

	def __init__(self,Mach,delta,positive,gamma=1.4):
		self.M= Mach
		self.d = delta
		self.g = gamma
		self.pos = positive
		
		import SalamanderSpace.Relations.ShockRelations as S
		from math import asin,pi
		self.mu = asin(1/self.M) *180/pi 
		
			
		
		self.nu = S.prandtlmeyer(None,self.M,self.g)
		self.phi = self.d + self.mu
		
		if self.pos == False:
			self.phi = self.d - self.mu
			
		
		self.K = self.nu - self.d




	def _printVars_(self):
		print("Mach:",self.M,"\nDelta:",self.d,"\nmu:",self.mu,"\nnu:",self.nu,"\nphi:",self.phi,"\nK:",self.K)
		print("\n\n")






"""
Name: Jason Saladiner Jr.
Date Created: 04/28/2022
Last Updated: 04/28/2022
Package Summary: 
	This package serves as a repository of fundamental relations found throughout Aerospace Engineering
Included Functions/Submodules:
	.Isentropic
	.Shock
"""



__all__ = ["Dimensionless", "Isentropic", "ShockRelations", "Thermodynamic"]